require 'json'
require 'ostruct'
require 'singleton'

module Spacelift
  # Policy is the module that hosts all the other resources in this library,
  # and provides helper methods to deal with the Collection singleton.
  module Policy
    module_function

    def define
      yield Collection.instance
    end

    def enforce(source)
      input = JSON.parse(source, object_class: OpenStruct)

      changes = input.resource_changes
      changes ? Collection.instance.process(changes) : []
    end

    # Collection is a singleton instance combining multiple rules. It's defined
    # this way to allow pulling policy from multiple files in no particular
    # order.
    class Collection
      include Singleton

      def initialize
        @rules = []
        @violations = []
      end

      def ensure(name, &block)
        raise Error, "definition not provided for rule '#{name}'" unless block

        @rules << Rule.new(name, &block)
      end

      def process(resources)
        raise Error, 'no rules defined' if @rules.empty?

        resources.each { |resource| process_resource(resource) }
        @violations
      end

      private

      def process_resource(resource)
        @rules.each do |rule|
          ok, violation = rule.process(resource)
          @violations << violation unless ok
        end
      end
    end
  end
end
