require 'colorize'
require 'set'

module Spacelift
  module Policy
    # Rule represents a single rule applied to all resources.
    class Rule
      attr_reader :name

      def initialize(name)
        @name = name
        @matchers = []
        @check = nil
        yield self
        freeze
        validate
      end

      def process(resource)
        return [true, nil] if ok?(resource)

        [false, Violation.new(address: resource.address, rule: name)]
      end

      def then(&block)
        raise Error, "check already defined on rule '#{name}'" if check

        self.check = block
      end

      def when(&block)
        matchers << block
        self
      end

      def when_action_is(*actions)
        required = Set.new(actions)

        self.when do |resource|
          Set.new(resource.change.actions).intersect?(required)
        end
      end

      def when_managed
        self.when { |resource| resource.mode == 'managed' }
      end

      def when_type_is(*types)
        self.when { |resource| types.include?(resource.type) }
      end

      private

      attr_accessor :matchers, :check

      def ok?(resource)
        return true unless matchers.all? { |matcher| matcher.call(resource) }

        change = resource.change
        check.call(change.before, change.after)
      end

      def validate
        raise Error, "no matchers defined on rule '#{name}'" if matchers.empty?
        raise Error, "no check defined on rule '#{name}'" unless check
      end
    end
  end
end
